@@ -284,6 +284,8 @@ def csr_decoder(isa_yaml, work_dir, bsv, logging=False):
     definefile.write(package_temp.format(pkg_name))
     definefile.write(func_decoder)
 
+    ro_csrs_addr = ''
+
     for node in isa_yaml['hart0']:
         cond_str = ''
         if isinstance(isa_yaml['hart0'][node], dict):
@@ -294,9 +296,14 @@ def csr_decoder(isa_yaml, work_dir, bsv, logging=False):
                     isa_yaml['hart0'][node]['address'] in user_trap_csrs:
                         cond_str = 'if ((misa[13]&misa[20]) == 1)'
 
+                if 'type' in isa_yaml['hart0'][node][rvxlen]:
+                    if 'ro_constant' in isa_yaml['hart0'][node][rvxlen]['type']:
+                        ro_csrs_addr += f'`{node.upper()} , '
+
                 csr_name = node
                 definefile.write(
                         case_address_temp.format(csr_name.upper(), cond_str))
+    ro_csrs_addr = ro_csrs_addr[:-2]
     definefile.write('\n           endcase')
     definefile.write('\n     return valid;')
     definefile.write('\n     endfunction')
@@ -319,6 +326,35 @@ def csr_decoder(isa_yaml, work_dir, bsv, logging=False):
       return ((prv >= csr_addr[9:8]) && !(write && csr_addr[11:10]==2'b11) );
     endfunction:fn_csrbox_access
 
+''')
+    definefile.write(f'''
+    
+    /*doc:func: This function controls if the pipeline should be flushed for
+    current csr op being decoded/processed. The control bits are used for
+    dynamic control and have the following behavior: 
+      - 00 : this is the default behavior where a flush is skipped for : read ops on all csrs that do not have read-side-effects (known at design time), writes on all read-only csrs
+      - 01 : in this configuration a flush is skipped for only writes to on read-only csrs
+      - 10 : in this configuration a flush is skipped for read-ops on all csrs do not have read-side-effects (known at design time)
+      - 11 : a flush is generated for all csr ops.
+    */
+    function Bool fn_csr_flush(Bit#(12) addr, Bit#(3) op, Bit#(5) rs1, Bit#(2) control);
+
+      Bool read_op = (op[1:0] == 2 || op[1:0] == 3) && rs1 == 0;
+      if (control == 'b11) // flush for all read and write ops
+        return True;
+      else if (control[1] == 1 && !read_op ) // flush on all write ops
+        return True;
+      else if (control[0] == 1 && read_op) // flush on all read ops
+        return True;
+      else if (read_op) // do not flush on reads of any csrs
+        return False;
+      else // do not flush on writes to read-only csrs. Flush for all other writes
+        case(addr)
+        {ro_csrs_addr}: return False;
+        default: return True;
+        endcase
+    endfunction:fn_csr_flush
+
 ''')
 
     definefile.write(end_package_temp)
